from django.utils.translation import gettext_lazy
from wagtail.admin.panels import Panel
from wagtail.admin.ui.side_panels import BaseSidePanel


class ReadingTimePanel(Panel):

    class SidePanel(BaseSidePanel):
        class SidePanelToggle(BaseSidePanel.SidePanelToggle):
            aria_label = gettext_lazy("Toggle reading time panel")
            icon_name = "rotate"
    
        name = "wagtail_reading_time"
        title = gettext_lazy("Reading Time")
        template_name = "wagtail_reading_time/panels/reading_time.html"
        order = 250
    
        def get_context_data(self, parent_context):
            context = super().get_context_data(parent_context)
            context["instance"] = self.object
            return context

    class BoundPanel(Panel.BoundPanel):
        template_name = "wagtail_reading_time/panels/reading_time.html"

        def get_context_data(self, parent_context=None):
            context = super().get_context_data(parent_context)
            context["instance"] = self.instance
            context["request"] = self.request
            return context

    




from django.db import models
from django.http import JsonResponse
from django.utils.html import strip_tags
from django.utils.translation import gettext_lazy as _, ngettext
from django.conf import settings
from wagtail.models import Page

import datetime


WPM = getattr(settings, "WAGTAIL_READING_TIME_WPM", 180)
WPS = WPM / 60



def estimate_reading_time(text: str) -> float:
    filtered_text = strip_tags(text)
    total_words = len(filtered_text.split())
    return total_words / WPS



class ReadingTimeMixin(models.Model):
    readime_time_fields = []

    _read_time = models.DurationField(
        default=datetime.timedelta(seconds=0),
        editable=False,
    )

    class Meta:
        abstract = True

    def _calculate_reading_time(self, fields = None):
        if fields is None:
            fields = self.readime_time_fields

        total_s = sum([estimate_reading_time(
            str(getattr(self, f))) 
            for f in self.readime_time_fields
        ])

        return datetime.timedelta(seconds=total_s)

    def save(self, *args, **kwargs):
        self._read_time = self._calculate_reading_time()
        super().save(*args, **kwargs)

    @property
    def reading_time(self):
        total_seconds = self._read_time.total_seconds()
        if total_seconds < 60:
            return _("Less than a minute")

        minutes = total_seconds // 60
        return ngettext(
            "%(minutes)d minute",
            "%(minutes)d minutes",
            minutes,
        ) % {"minutes": minutes}


class PageReadingTimeMixin(ReadingTimeMixin, Page):

    def serve_preview(self, request, mode_name):
        
        self._read_time = self._calculate_reading_time()

        if mode_name == "reading_time":
            return JsonResponse({
                "reading_time": self.reading_time,
            }, status=200)

        return super().serve_preview(request, mode_name)

    class Meta:
        abstract = True
        


from wagtail import hooks
from django.templatetags.static import static
from django.utils.html import format_html

@hooks.register('insert_global_admin_css')
def global_admin_css():
    return format_html(
        '<link rel="stylesheet" href="{}">',
        static('wagtail_reading_time/panels/reading_time.css')
    )

@hooks.register('insert_global_admin_js')
def global_admin_js():
    return format_html(
        '<script src="{}"></script>',
        static('wagtail_reading_time/panels/reading_time.js')
    )
